% Copyright: Stefan  M. Karlsson, Josef Bigun 2014
function [U, V] = DoFlowLK(dx, dy, dt,flowRes)
% function DoFlow inputs images, dr, dy, dt, corresponding to the
% 3D gradients of a video feed
if nargin < 4
    flowRes = size(dx)/8;end

stdTensor = 1.8; 
gg  = single(gaussgen(stdTensor)); %% filter for tensor smoothing

U = zeros(flowRes,'single');
V = zeros(flowRes,'single');

%     MOMENT CALCULATIONS
%     moment m200, calculated in 3 steps explicitly
%     1) make elementwise product
      momentIm = dx.^2;
     
%     2) smooth with large seperable gaussian filter (spatial integration)
      momentIm = conv2(gg,gg,momentIm,'same');

%     3) downsample to specified resolution (imresizeNN function is found in "helperFunctions"):     
      m200 =  imresizeNN(momentIm ,flowRes);
      
%    The remaining moments are calculated in EXACTLY the same way as above, condensed to one liners:
 m020=imresizeNN(conv2(gg,gg, sum(dy.^2 ,3),'same'),flowRes);
 m110=imresizeNN(conv2(gg,gg, sum(dx.*dy,3),'same'),flowRes);
 m101=imresizeNN(conv2(gg,gg, sum(dx.*dt,3),'same'),flowRes);
 m011=imresizeNN(conv2(gg,gg, sum(dy.*dt,3),'same'),flowRes);

 %Implement the vectorized formulation of the solver:
 denominator=(m020.*m200 - m110.^2);
 % Threshold TO AVOID DIVISION WITH ZERO denominator:
 EPSILONLK = 0.3;
 valid= EPSILONLK<denominator;
 U(valid) =(m101(valid).*m110(valid) - m011(valid).*m200(valid))./denominator(valid);
 V(valid) =-(-m101(valid).*m020(valid) + m011(valid).*m110(valid))./denominator(valid);
 
 valid= 16 <U.*U+V.*V; %Delete velocities faster than 4 pixels/time-unit 
U(valid)=0; V(valid)=0;