function [U,V, OfErr] = correlation(imNew,bFineScale,bInitialize,flowRes)
% Declare  persistent variables, these will remain between calls
persistent siz W H Nhm Nhh dxh dyh y0_range_all x0_range_all imPrev imPrev_ ...
    PrevNorm imNew_ OfErrAll OfNormAll onemask;

Nhm=3;  %The PATTERN (Mönster in swedish) defining a unique point has the size 2*Nhm+1 x 2*Nhm+1
Nhh=3; %Nhh defines allowed locations to which to displace the pattern, i.e.
%pixel locations  within the  2*Nhh+1 x 2*Nhh+1 square. Nhh is independent of Nhm,
%it can be smaller than, equal to or larger than Nhm. Although, the  2*Nhh+1 x 2*Nhh+1 square
%is the SEARCH AREA (for optical flow vectors), all pixels within  a larger square,
%the 2*(Nhh+Nhm)+1 x 2*(Nhh+Nhm)+1 contribute to determine U, V, the optical flow.
%The 2*(Nhh+Nhm)+1 x 2*(Nhh+Nhm)+1 square is the SUPPORT AREA of
%the optical flow. Support Area for Nhm=3, and Nhh=4, 15x15 sized.
%Then the 7x7 sized pattern can be moved around fully INSIDE a 15x15 support area
%to provide 81 (9x9, sized search area) different optical flow vectors.
Thr_std_err=0.01;

smask=(2*Nhh+1)^2;

% initialize persistent variables, these will remain between calls
if nargin>2 && bInitialize
    %Do nothing before updating size of the image
    if bFineScale
        %if we will use a fine scale for derivatives, then we set
        %the size to be that of the original video
        siz = size(imNew);
        W= siz(2);
        H= siz(1);
        
    else% if ~bFineScale
        %if we use a coarse scale, set the size to half of the original:
        siz = floor(size(imNew)/2);
        W= siz(2);
        H= siz(1);
        
    end
        %imPrev will contain the previous image, we use single precision for
        %faster calculations
        imPrev= single(imNew);
        PrevNorm=single(zeros(H,W));
        imPrev_=zeros([H W]+2*(Nhh+Nhm)*[1 1]);
        imNew_=zeros([H W]+2*(Nhh+Nhm)*[1 1]);
        onemask=ones(2*Nhm+1,1);
        
        [dxh,dyh]=meshgrid([-Nhh:Nhh],-[-Nhh:Nhh]);
        
        OfErrAll=single(zeros([size(imNew)+ [2*(Nhh+Nhm) 2*(Nhh+Nhm)], smask]));
        for id=1:smask
            y0_range_all(:,id)=[1+Nhh+Nhm:H+Nhh+Nhm]'-dyh(id);
            x0_range_all(:,id)=[1+Nhh+Nhm:W+Nhh+Nhm] -dxh(id);
            OfNormAll=single(zeros([size(imNew)+ [2*(Nhh+Nhm) 2*(Nhh+Nhm)], smask]));
        end
end

%%% resample image to right resolution, depending on the scale we chose:
if ~bFineScale
    imNew = imresizeNN(conv2(single(imNew),gg,'same'),siz);
else
    imNew = single(imNew);
end

%%%% calculate the derivatives explicitly. We use small filters here, so it
%%%% makes sense to use explicit convolutions.
%%%% TODO, implement the derivatives by convolutions. Use the conv2 function
%%%% as it is the fastest


imPrev_((Nhh+Nhm+1):(H+Nhh+Nhm) , (Nhh+Nhm+1):(W+Nhh+Nhm))=imPrev;

% Subimages of the new frame will be correlated against the prevIm
imNew_((Nhh+Nhm+1):(H+Nhh+Nhm) , (Nhh+Nhm+1):(W+Nhh+Nhm))=imNew;


U = zeros(flowRes,'single');
V=U;

for id=1:smask %Visit all locations within the Search mask
    OfErrAll(y0_range_all(:,id),x0_range_all(:,id),id)= imNew; %PRODUCE \tilde f of eq. 12.108 of the book.
    %For each id we produce one \tilde f, the displaced frame. Each \tilde f is a shifted version of the same frame to see
    %which shift agrees best with the previous image.  Displacement vectors are drawn from locations of the search window
    
    OfNormAll(:,:,id)=FILL_IN_PLEASE; %Compute pointwise square of displaced frame
    OfNormAll(:,:,id)=FILL_IN_PLEASE; %produce the norm using filtering by onemask ,
    
    OfErrAll(:,:,id)=FILL_IN_PLEASE;% Build the pointwise product between the previous frame and  displaced frames...
    OfErrAll(:,:,id)=filter2(onemask',    filter2(onemask,OfErrAll(:,:,id))   ); %produce the squared norm OF  the scalar products using filtering and onemask, 
end

if ~bInitialize
    for id=1:smask OfErrAll(:,:,id)=OfErrAll(:,:,id)./(PrevNorm.*OfNormAll(:,:,id)+0.01); end  %Build the similarity score (Schwartz inequality)
end

[OfErr,idmx]=max(OfErrAll,[],3); %We identify the shift yielding the smallest squared error and note it in the image idmx, the shift-identity image...
Err_std=std(OfErrAll,0,3); %We find the standard deviation of all errors to later skip "nearly constant" areas from displaying as these will be unstable.
idmx_fr=imresizeNN(idmx,flowRes); %We reduce the shift-identiy  image...
Err_std_fr=imresizeNN(Err_std,flowRes);%and the standard deviation error image

non_trivial=(Thr_std_err<Err_std_fr); %We skip the "nearly constant" areas

U(non_trivial)=dxh(idmx_fr(non_trivial))/Nhh; %We limit speed vectors for graphical presentation
V(non_trivial)=dyh(idmx_fr(non_trivial))/Nhh; %We limit speed vectors for graphical presentation

%%% resample image to right resolution, depending on the scale we chose:
if ~bFineScale
    imNew = imresizeNN(conv2(single(imNew),gg,'same'),siz);
else
    imNew = single(imNew);
end




% finally, store away the current image for use on next frame
imPrev = imNew;
PrevNorm= OfNormAll(:,:,(smask+1)/2);
return

