function [U,V, OfErr] = dfd(imNew,bFineScale,bInitialize,flowRes)
% Declare  persistent variables, these will remain between calls
persistent siz W H Nhm Nhh dxh dyh y0_range_all x0_range_all imPrev imPrev_ ...
    PrevSqNorm imNew_ OfErrAll OfSqNormAll onemask;

Nhm=3;  %The PATTERN (Mönster in swedish) defining a unique point has the size 2*Nhm+1 x 2*Nhm+1
Nhh=3; %Nhh defines allowed locations to which to displace the pattern, i.e.
%pixel locations  within the  2*Nhh+1 x 2*Nhh+1 square. Nhh is independent of Nhm,
%it can be smaller than, equal to or larger than Nhm. Although, the  2*Nhh+1 x 2*Nhh+1 square
%is the SEARCH AREA (for optical flow vectors), all pixels within  a larger square,
%the 2*(Nhh+Nhm)+1 x 2*(Nhh+Nhm)+1 contribute to determine U, V, the optical flow.
%The 2*(Nhh+Nhm)+1 x 2*(Nhh+Nhm)+1 square is the SUPPORT AREA of
%the optical flow. Support Area for Nhm=3, and Nhh=4, 15x15 sized.
%Then the 7x7 sized pattern can be moved around fully INSIDE a 15x15 support area
%to provide 81 (9x9, sized search area) different optical flow vectors.
Thr_std_err=10;

smask=(2*Nhh+1)^2;

Thr_std_err=(Thr_std_err^2/smask);

% initialize persistent variables, these will remain between calls
if nargin>2 && bInitialize
    %This part is executed only ONCE, at the beginning.    

    %Updating size of the image if the user requests has requested it
    if bFineScale
        %if we will use a fine scale for derivatives, then we set
        %the size to be that of the original video
        siz = size(imNew);
        W= siz(2);
        H= siz(1);
    else% if ~bFineScale   %Thi is essentially the same as the one above except that it is repeated 
        %if we use a coarse scale, set the size to half of the original:
        siz = floor(size(imNew)/2);
        W= siz(2);
        H= siz(1);
    end
        
        %imPrev will contain the previous image, we use single precision for
        %faster calculations
        imPrev= single(imNew);
        imPrev_=single(zeros([H W]+2*(Nhh+Nhm)*[1 1]));
        imNew_=single(zeros([H W]+2*(Nhh+Nhm)*[1 1]));
        onemask=single(ones(2*Nhm+1,1);
        
        [dxh,dyh]=meshgrid([-Nhh:Nhh],-[-Nhh:Nhh]);
        
        OfErrAll=single(zeros([size(imNew)+ [2*(Nhh+Nhm) 2*(Nhh+Nhm)], smask]));
        for id=1:smask
            y0_range_all(:,id)=[1+Nhh+Nhm:H+Nhh+Nhm]'-dyh(id); %Determine which range of rows will be moved up or down 
            x0_range_all(:,id)= FILL_IN_PLEASE       %Determine which range of columns will be moved left or right
        end
end

%%% resample image to right resolution, depending on the scale we chose:
if ~bFineScale
    imNew = imresizeNN(conv2(single(imNew),gg,'same'),siz);
else
    imNew = single(imNew);
end

%%%% calculate the derivatives explicitly. We use small filters here, so it
%%%% makes sense to use explicit convolutions.
%%%% TODO, implement the derivatives by convolutions. Use the conv2 function
%%%% as it is the fastest


imPrev_((Nhh+Nhm+1):(H+Nhh+Nhm) , (Nhh+Nhm+1):(W+Nhh+Nhm))=imPrev;

% Subimages of the new frame will be correlated against the prevIm
imNew_((Nhh+Nhm+1):(H+Nhh+Nhm) , (Nhh+Nhm+1):(W+Nhh+Nhm))=imNew;


U = zeros(flowRes,'single');
V=U;

for id=1:smask %Visit all locations within the Search mask
    OfErrAll(y0_range_all(:,id),FILL_IN_PLEASE,id) = imNew; %PRODUCE \tilde f of eq. 12.108 of the book.
    %For each id we produce one \tilde f, the displaced frame. Even if each   
    %\tilde f is a shifted version of the entire frame, the net effect is  that all local neighgorhoods are shifted which allows to check 
    %all errors of all neighborhoods having ONE and SAME shift at a time,  to gain computational speed.  This is done next.
    
    OfErrAll(:,:,id)=OfErrAll(:,:,id)- imPrev_;% The error of each shift in comparison to  previous frame replaces pixel values...
    OfErrAll(:,:,id)=FILL_IN_PLEASE; %squared errors replaces pixel values
    OfErrAll(:,:,id)=filter2(FILL_IN_PLEASE,    filter2(FILL_IN_PLEASE,OfErrAll(:,:,id))   ); %produce the squared norm OF THE ERROR, i.e. eq.12.108
    %hINT...USE onemask to sum up the squares..via filter2.
    
 
end

[OfErr,idmx]=min(OfErrAll,[],3); %We identify the shift yielding the smallest squared error in each neighborhood
                                 %and note the identity in the matrix idmx, which can also viewed as an image...

Err_std=std(OfErrAll,0,3); %We find the standard deviation of all errors to later skip "nearly constant" areas from displaying 
                           %as we know for sure that in these neighborhoods optical flow computation will not be reliable at all.
                           
                           
idmx_fr=imresizeNN(idmx,flowRes); %We reduce the shift-identiy  image...
Err_std_fr=imresizeNN(Err_std,flowRes);%and the standard deviation error image

non_trivial=Thr_std_err<Err_std_fr; %We skip the "nearly constant" areas after size reduction of the error image.
U(non_trivial)=dxh(idmx_fr(non_trivial))/Nhh; %We limit speed vectors for graphical presentation by making them at most 1 in each component
V(non_trivial)=dyh(idmx_fr(non_trivial))/Nhh; %We limit speed vectors for graphical presentation...

%%% resample image to right resolution, depending on the scale we chose:
if ~bFineScale
    imNew = imresizeNN(conv2(single(imNew),gg,'same'),siz);
else
    imNew = single(imNew);
end




% finally, store away the current image for use on next frame
imPrev = imNew;
PrevSqNorm= OfSqNormAll(:,:,(smask+1)/2);
return

